@@ -50,20 +50,15 @@ SOFTWARE.
  *
  */
 
-#define	 NEED_EVENTS	/* for inputstr.h    */
-#define	 NEED_REPLIES
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
 #endif
 
-#include <X11/X.h>	/* for inputstr.h    */
-#include <X11/Xproto.h>	/* Request macro     */
-#include "inputstr.h"	/* DeviceIntPtr      */
+#include "inputstr.h"           /* DeviceIntPtr      */
 #include <X11/extensions/XI.h>
+#include <X11/extensions/XI2.h>
 #include <X11/extensions/XIproto.h>
 #include "exevents.h"
-#include "extnsionst.h"
-#include "extinit.h"	/* LookupDeviceIntRec */
 #include "exglobals.h"
 
 #include "setmmap.h"
@@ -78,10 +73,8 @@ SOFTWARE.
 int
 SProcXSetDeviceModifierMapping(ClientPtr client)
 {
-    char n;
-
     REQUEST(xSetDeviceModifierMappingReq);
-    swaps(&stuff->length, n);
+    swaps(&stuff->length);
     return (ProcXSetDeviceModifierMapping(client));
 }
 
@@ -97,38 +90,40 @@ ProcXSetDeviceModifierMapping(ClientPtr client)
     int ret;
     xSetDeviceModifierMappingReply rep;
     DeviceIntPtr dev;
-    KeyClassPtr kp;
 
     REQUEST(xSetDeviceModifierMappingReq);
     REQUEST_AT_LEAST_SIZE(xSetDeviceModifierMappingReq);
 
-    dev = LookupDeviceIntRec(stuff->deviceid);
-    if (dev == NULL) {
-	SendErrorToClient(client, IReqCode, X_SetDeviceModifierMapping, 0,
-			  BadDevice);
-	return Success;
-    }
+    if (stuff->length != bytes_to_int32(sizeof(xSetDeviceModifierMappingReq)) +
+        (stuff->numKeyPerModifier << 1))
+        return BadLength;
 
-    rep.repType = X_Reply;
-    rep.RepType = X_SetDeviceModifierMapping;
-    rep.length = 0;
-    rep.sequenceNumber = client->sequence;
+    rep = (xSetDeviceModifierMappingReply) {
+        .repType = X_Reply,
+        .RepType = X_SetDeviceModifierMapping,
+        .sequenceNumber = client->sequence,
+        .length = 0
+    };
 
-    ret = SetModifierMapping(client, dev, stuff->length,
-			     (sizeof(xSetDeviceModifierMappingReq) >> 2),
-			     stuff->numKeyPerModifier, (BYTE *) & stuff[1],
-			     &kp);
+    ret = dixLookupDevice(&dev, stuff->deviceid, client, DixManageAccess);
+    if (ret != Success)
+        return ret;
+
+    ret = change_modmap(client, dev, (KeyCode *) &stuff[1],
+                        stuff->numKeyPerModifier);
+    if (ret == Success)
+        ret = MappingSuccess;
 
     if (ret == MappingSuccess || ret == MappingBusy || ret == MappingFailed) {
-	rep.success = ret;
-	if (ret == MappingSuccess)
-	    SendDeviceMappingNotify(client, MappingModifier, 0, 0, dev);
-	WriteReplyToClient(client, sizeof(xSetDeviceModifierMappingReply),
-			   &rep);
-    } else {
-	if (ret == -1)
-	    ret = BadValue;
-	SendErrorToClient(client, IReqCode, X_SetDeviceModifierMapping, 0, ret);
+        rep.success = ret;
+        WriteReplyToClient(client, sizeof(xSetDeviceModifierMappingReply),
+                           &rep);
+    }
+    else if (ret == -1) {
+        return BadValue;
+    }
+    else {
+        return ret;
     }
 
     return Success;
@@ -143,11 +138,9 @@ ProcXSetDeviceModifierMapping(ClientPtr client)
 
 void
 SRepXSetDeviceModifierMapping(ClientPtr client, int size,
-			      xSetDeviceModifierMappingReply * rep)
+                              xSetDeviceModifierMappingReply * rep)
 {
-    char n;
-
-    swaps(&rep->sequenceNumber, n);
-    swapl(&rep->length, n);
-    WriteToClient(client, size, (char *)rep);
+    swaps(&rep->sequenceNumber);
+    swapl(&rep->length);
+    WriteToClient(client, size, rep);
 }
